@@ -7,11 +7,10 @@ from twitterhal.engine import TwitterHAL
 from twitterhal.runtime import runner
 
 
-def init_logging(loglevel=logging.ERROR):
-    logging.basicConfig(
-        format="%(asctime)s: [%(funcName)s: %(lineno)d] %(message)s", level=loglevel, datefmt="%H:%M:%S"
-    )
-    return logging.getLogger(__package__)
+logging.basicConfig(
+    format="%(asctime)s: [%(funcName)s: %(lineno)d] %(message)s", level=logging.INFO, datefmt="%H:%M:%S"
+)
+logger = logging.getLogger("twitterhal")
 
 
 class CommandLine:
@@ -45,10 +44,6 @@ class CommandLine:
         self.mutex.add_argument("--stats", action="store_true", help="Display some stats")
         self.mutex.add_argument("--print-config", action="store_true", help="Print current parsed config")
         self.mutex.add_argument("--post-random", action="store_true", help="Post a new random tweet")
-        self.mutex.add_argument(
-            "--mark-mentions-answered", action="store_true",
-            help="Fetch the latest mentions and mark them all as answered. Useful if you had to re-init the DB."
-        )
         self.mutex.add_argument(
             "--version", action="version", version="%(prog)s " + __version__,
             help="Show program's version number and exit"
@@ -56,65 +51,55 @@ class CommandLine:
 
     def __enter__(self):
         self.setup()
+        self.hal = self.TwitterHAL(**self.get_hal_kwargs())
+        self.hal.open()
         return self
 
     def __exit__(self, *args, **kwargs):
-        pass
+        self.hal.close()
 
     def setup(self, *args, **kwargs):
         self.args = self.parser.parse_args()
         settings.setup(settings_module=self.args.settings_module)
         if self.args.debug:
-            logger = init_logging(logging.DEBUG)
+            logger.setLevel(logging.DEBUG)
             logger.debug("TESTING DEBUG LOGGING")
-        elif self.args.run:
-            logger = init_logging(logging.INFO)
-        else:
-            logger = init_logging()
-        self.init_megahal = self.args.run or self.args.chat or self.args.post_random
 
     def get_hal_kwargs(self):
-        return {"init_megahal": self.init_megahal, "force": self.args.force, "test": self.args.test}
+        return {"force": self.args.force, "test": self.args.test}
 
     def run(self, *args, **kwargs):
         if self.args.chat:
-            with self.TwitterHAL(**self.get_hal_kwargs()) as hal:
-                hal.megahal.interact()
+            self.hal.megahal.interact()
         elif self.args.stats:
-            with self.TwitterHAL(**self.get_hal_kwargs()) as hal:
-                self.print_stats(hal)
+            self.print_stats()
         elif self.args.print_config:
             print(settings)
         elif self.args.post_random:
-            with self.TwitterHAL(**self.get_hal_kwargs()) as hal:
-                hal.post_random_tweet()
-        elif self.args.mark_mentions_answered:
-            with self.TwitterHAL(**self.get_hal_kwargs()) as hal:
-                hal.mark_mentions_answered()
+            self.hal.post_random_tweet()
         elif self.args.run:
-            with self.TwitterHAL(**self.get_hal_kwargs()) as hal:
-                runner.sleep_seconds = settings.RUNNER_SLEEP_SECONDS
-                runner.run()
+            self.hal.prepare_runner()
+            runner.sleep_seconds = settings.RUNNER_SLEEP_SECONDS
+            runner.run()
         elif not self.run_extra():
             self.parser.print_help()
 
     def run_extra(self, *args, **kwargs):
         """
         Plug in your extra routines here. Make sure this returns True if any
-        of them were applicable and run. And wrap them in
-        `self.TwitterHAL(**self.get_hal_kwargs()) as hal` if so is required.
+        of them were applicable and run.
         """
         return False
 
-    def print_stats(self, hal, **kwargs):
-        print("Posted random tweets:         %d" % len(hal.db.posted_tweets.original_posts))
-        print("  - earliest date:            %s" % hal.db.posted_tweets.original_posts.earliest_date)
-        print("  - latest date:              %s" % hal.db.posted_tweets.original_posts.latest_date)
-        print("Posted reply tweets:          %d" % len(hal.db.posted_tweets.replies))
-        print("  - earliest date:            %s" % hal.db.posted_tweets.replies.earliest_date)
-        print("  - latest date:              %s" % hal.db.posted_tweets.replies.latest_date)
-        print("Mentions:                     %d" % len(hal.db.mentions))
-        print("  - unanswered:               %d" % len(hal.db.mentions.unanswered))
+    def print_stats(self):
+        print("Posted random tweets:         %d" % len(self.hal.db.posted_tweets.original_posts))
+        print("  - earliest date:            %s" % self.hal.db.posted_tweets.original_posts.earliest_date)
+        print("  - latest date:              %s" % self.hal.db.posted_tweets.original_posts.latest_date)
+        print("Posted reply tweets:          %d" % len(self.hal.db.posted_tweets.replies))
+        print("  - earliest date:            %s" % self.hal.db.posted_tweets.replies.earliest_date)
+        print("  - latest date:              %s" % self.hal.db.posted_tweets.replies.latest_date)
+        print("Mentions:                     %d" % len(self.hal.db.mentions))
+        print("  - unanswered:               %d" % len(self.hal.db.mentions.unanswered))
 
 
 def main():
