@@ -89,6 +89,12 @@ class Settings:
         if self.SCREEN_NAME == "":
             warnings.warn("settings.SCREEN_NAME cannot be empty")
 
+        if self.DETECTLANGUAGE_API_KEY:
+            try:
+                import detectlanguage  # noqa
+            except ImportError:
+                warnings.warn("settings.DETECTLANGUAGE_API_KEY was set, but detectlanguage could not be imported")
+
         self.is_setup = True
 
     def __setattr__(self, key, value):
@@ -116,7 +122,7 @@ class Settings:
         return getattr(self, key, default)
 
     def get_database_class(self):
-        mod, klass = self.DATABASE_CLASS.rsplit(".", maxsplit=1)
+        mod, klass = self.DATABASE["class"].rsplit(".", maxsplit=1)
         return getattr(importlib.import_module(mod), klass)
 
 
