@@ -17,6 +17,7 @@ from twitterhal.conf import settings
 from twitterhal.gracefulkiller import killer
 from twitterhal.models import Tweet, TweetList
 from twitterhal.runtime import runner
+from twitterhal.twitter_api import TwitterApi
 
 
 logger = logging.getLogger(__name__)
@@ -25,7 +26,7 @@ logger = logging.getLogger(__name__)
 class TwitterHAL:
     def __init__(
         self, screen_name=None, random_post_times=None, include_mentions=False,
-        init_megahal=False, force=False, test=False, **kwargs
+        force=False, test=False, **kwargs
     ):
         """Initialize the bot.
 
@@ -41,22 +42,22 @@ class TwitterHAL:
             include_mentions (bool, optional): Whether to include all
                 @mentions in our replies, and not only that of the user we're
                 replying to. Default: False
-            init_megahal (bool, optional): Initialize MegaHAL right at the
-                beginning and not on first access. Default: False
             force (bool, optional): Try and force actions even if TwitterHAL
                 doesn't want to. Default: False
             test (bool, optional): Test mode; don't actually post anything.
                 Default: False
         """
         # Take care of settings
-        db_path = "test" if test else settings.DATABASE_FILE
         Database = settings.get_database_class()
+        db_options = settings.DATABASE.get("options", {})
+        if test:
+            db_options.update(settings.DATABASE.get("test_options", {}))
         self.include_mentions = include_mentions or settings.INCLUDE_MENTIONS
         self.random_post_times = random_post_times or settings.RANDOM_POST_TIMES
         self.screen_name = screen_name or settings.SCREEN_NAME
 
         # Set up runtime stuff
-        self.db = cast("DBInstance", Database(db_path=db_path))
+        self.db = cast("DBInstance", Database(**db_options))
         self.force = force
         self.generate_random_lock = threading.Lock()
         self.megahal_open = False
@@ -65,8 +66,6 @@ class TwitterHAL:
         self.test = test
         if self.test:
             logger.info("TEST MODE")
-        if init_megahal:
-            self.megahal
 
     """ ---------- METHODS FOR SETTING UP STUFF ---------- """
 
@@ -76,32 +75,41 @@ class TwitterHAL:
 
     def __exit__(self, *args, **kwargs):
         self.close()
-        logger.info("Exited gracefully =)")
+        logger.debug("Exited gracefully =)")
 
     def open(self):
         """
-        Will chicken out if Twitter authentication fails. This is intentional.
+        Will *not* chicken out if Twitter authentication fails. But it will
+        happen sooner or later.
         """
-        logger.info("Starting engine ...")
-        logger.debug("Initializing Twitter API ...")
-        self.api = twitter.Api(**self.get_twitter_api_kwargs())
-        self.api.InitializeRateLimit()
+        logger.debug("Starting engine ...")
         self.init_db()
+        logger.debug("Initializing Twitter API ...")
+        self.api = TwitterApi(**self.get_twitter_api_kwargs())
+        try:
+            self.api.InitializeRateLimit()
+        except twitter.TwitterError:
+            pass
         self._init_post_status_limit()
+        logger.debug("Ready!")
+
+    def close(self):
+        logger.debug("Closing DB ...")
+        self.db.close()
+        if self.megahal_open:
+            logger.debug("Closing MegaHAL ...")
+            self.megahal.close()
+            self.megahal_open = False
+
+    def prepare_runner(self):
+        self._get_missing_own_tweets()
+        self._get_missing_mentions()
+        self._flag_replied_mentions()
         for mention in self.db.mentions.unanswered:
             self.mention_queue.put(mention)
         self.register_workers()
         self.register_loop_tasks()
         self.register_post_loop_tasks()
-        logger.info("Ready!")
-
-    def close(self):
-        logger.info("Closing DB ...")
-        self.db.close()
-        if self.megahal_open:
-            logger.info("Closing MegaHAL ...")
-            self.megahal.close()
-            self.megahal_open = False
 
     def register_workers(self):
         runner.register_worker(self.post_tweets_worker)
@@ -132,19 +140,13 @@ class TwitterHAL:
         If you extend the functionality and want to store additional values in
         the DB, this is the place to define them. Just do:
 
-        >>> self.db.add_key(key_name, key_type, key_default)
+        >>> self.db.add_key(key_name, key_type, **default_kwargs)
         >>> super().init_db()
         """
+        self.db.add_key("posted_tweets", TweetList, unique=True)
+        self.db.add_key("mentions", TweetList, unique=True)
         logger.debug("Trying to initialize DB ...")
         self.db.open()
-        # Just for safety:
-        try:
-            random_tweets = self.api.GetUserTimeline(screen_name=self.screen_name, exclude_replies=True, count=1)
-        except twitter.TwitterError:
-            warnings.warn("Could not connect to Twitter API! Keys/secrets incorrect?")
-            random_tweets = []
-        if len(random_tweets) > 0 and random_tweets[0] not in self.db.posted_tweets:
-            self.db.posted_tweets.append(Tweet.from_status(random_tweets[0]))
         logger.debug("DB initialized")
 
     @property
@@ -167,13 +169,13 @@ class TwitterHAL:
         while not killer.kill_now or not self.post_queue.empty():
             if self.force or self.can_post():
                 try:
-                    tweet: "Tweet" = self.post_queue.get(timeout=1)
+                    tweet: "Tweet" = self.post_queue.get(timeout=3)
                 except queue.Empty:
-                    # Of course it *shouldn't* be empty now, but we don't want
-                    # anything to fail if we can help it
                     continue
                 logger.debug(f"Got from post_queue: {tweet}")
                 self._post_tweet(tweet)
+            else:
+                killer.sleep(5)
         logger.debug("Recevied exit event")
 
     """ ---------- LOOP TASKS ---------- """
@@ -339,8 +341,9 @@ class TwitterHAL:
             else:
                 logger.info(f"Got duplicate reply, trying again (since {start_time}): {reply}")
             reply = self.megahal.get_reply_nolearn(phrase, max_length=CHARACTER_LIMIT - len(prefix))
+        text = prefix + reply.text + suffix
         tweet = Tweet(
-            text=prefix + reply.text + suffix,
+            text=text, filtered_text=text,
             in_reply_to_status_id=in_reply_to.id if in_reply_to is not None else None
         )
         logger.debug(f"Generated: {tweet}")
@@ -352,16 +355,6 @@ class TwitterHAL:
 
     """ ---------- HELPFUL (?) UTILITY METHODS ---------- """
 
-    def mark_mentions_answered(self):
-        """Fetch all recent mentions and mark them as answered"""
-        self.get_new_mentions()
-        count = 0
-        while not self.mention_queue.empty():
-            mention: "Tweet" = self.mention_queue.get()
-            mention.is_answered = True
-            count += 1
-        logger.info(f"Fetched {count} mentions and marked them as answered")
-
     def post_from_queue(self):
         """Post all queued Tweets
 
@@ -388,6 +381,33 @@ class TwitterHAL:
 
     """ ---------- PRIVATE HELPER METHODS ---------- """
 
+    def _get_missing_mentions(self):
+        logger.info("Fetching mentions ...")
+        try:
+            since_id = max([t.id for t in self.db.mentions])
+        except ValueError:
+            since_id = None
+        tweets = self.api.GetMentions(since_id=since_id, count=200)
+        self.db.mentions.extend([Tweet.from_status(t) for t in tweets])
+
+    def _get_missing_own_tweets(self):
+        logger.info("Fetching own posted tweets ...")
+        try:
+            since_id = max([t.id for t in self.db.posted_tweets])
+        except ValueError:
+            since_id = None
+        tweets = self.api.GetUserTimeline(screen_name=self.screen_name, since_id=since_id, count=200)
+        self.db.posted_tweets.extend([Tweet.from_status(t) for t in tweets])
+
+    def _flag_replied_mentions(self):
+        # Make sure _get_missing_mentions() and _get_missing_own_tweets() is
+        # run *before* this one
+        logger.info("Flagging replied mentions ...")
+        in_reply_to_ids = [t.in_reply_to_status_id for t in self.db.posted_tweets.replies]
+        for mention in [t for t in self.db.mentions.unanswered if t.id in in_reply_to_ids]:
+            mention.is_answered = True
+        self.db.sync("mentions")
+
     def _init_post_status_limit(self):
         """Initialize status/retweet post limit data
 
@@ -433,12 +453,14 @@ class TwitterHAL:
             # Logging the request here, since I guess it counts towards
             # the rate limit regardless of whether we succeed or not
             self._set_post_status_limit(subtract=1)
-            self.db.posted_tweets.append(Tweet.from_status(status))
+            tweet = Tweet.from_status(status)
+            self.db.posted_tweets.append(tweet)
             if tweet.in_reply_to_status_id:
                 # This was a reply to a mention
                 original_tweet = self.db.mentions.get_by_id(tweet.in_reply_to_status_id)
                 if original_tweet:
                     original_tweet.is_answered = True
+                    self.db.sync("mentions")
                 logger.info(f"Posted: {tweet} as reply to: {original_tweet}")
             else:
                 logger.info(f"Posted: {tweet}")
